/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.util.Utility;

/** The menu bar for the Gatherer main GUI. 
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.2
 */
public class MenuBar
    extends JMenuBar {
    /** The icon to be displayed alongside the context choosen help file. */
    private int current_tab          = -1;

    private JMenu file               = null;
    private JMenu edit               = null;
    private JMenu metadata           = null;
    private JMenu tools              = null;
    public JMenu help                = null;
    public JMenuItem file_associations;
    public JMenuItem file_close      = null;
    public JMenuItem file_delete     = null;
    public JMenuItem file_exit       = null;
    public JMenuItem file_export     = null;
    public JMenuItem file_new        = null;
    public JMenuItem file_open       = null;
    public JMenuItem file_options    = null;
    public JMenuItem file_save       = null;
    public JMenuItem file_save_as;
    public JMenuItem edit_copy;
    public JMenuItem edit_cut;
    public JMenuItem edit_paste;
    public JMenuItem edit_undo       = null;
    public JMenuItem edit_redo       = null;
    public JMenuItem edit_replace = null;
    public JMenuItem edit_search = null;
    public JMenuItem metadata_import = null;
    public JMenuItem metadata_edit   = null;
    public JMenuItem metadata_export = null;
    public JMenuItem tools_log       = null;
    public JMenuItem tools_size      = null;
    public JMenuItem help_about;
    public JMenuItem help_browse;
    public JMenuItem help_build;
    public JMenuItem help_collect;
    public JMenuItem help_design;
    public JMenuItem help_export;
    public JMenuItem help_general;
    public JMenuItem help_metaedit;
    public JMenuItem help_mirror ;
    public JMenuItem help_preview;
    public MagicMenuItem metadata_view   = null;

    public MenuBar(MenuListener menu_listener)
    {
	file = new JMenu();
	file.setMnemonic(KeyEvent.VK_F);
	Dictionary.registerText(file, "Menu.File");

	file_associations = new JMenuItem();
	file_associations.addActionListener(Gatherer.g_man);	  
	file_associations.setMnemonic(KeyEvent.VK_A);
	Dictionary.registerText(file_associations, "Menu.File_Associations");

	file_close = new JMenuItem();
	file_close.addActionListener(Gatherer.g_man);
	file_close.setEnabled(false);
	file_close.setMnemonic(KeyEvent.VK_C);
	Dictionary.registerText(file_close, "Menu.File_Close");

	file_delete = new JMenuItem();
	file_delete.addActionListener(Gatherer.g_man);
	file_delete.setMnemonic(KeyEvent.VK_D);
	Dictionary.registerText(file_delete, "Menu.File_Delete");

	file_exit = new JMenuItem();
	file_exit.addActionListener(Gatherer.g_man);
	file_exit.setMnemonic(KeyEvent.VK_X);
	Dictionary.registerText(file_exit, "Menu.File_Exit");

	file_export = new JMenuItem();
	file_export.addActionListener(Gatherer.g_man);
	file_export.setMnemonic(KeyEvent.VK_E);
	Dictionary.registerText(file_export, "Menu.File_Export");

	file_new = new JMenuItem();
	file_new.addActionListener(Gatherer.g_man);
	file_new.setMnemonic(KeyEvent.VK_N);
	Dictionary.registerText(file_new, "Menu.File_New");

	file_open = new JMenuItem();
	file_open.addActionListener(Gatherer.g_man);
	file_open.setMnemonic(KeyEvent.VK_O);
	Dictionary.registerText(file_open, "Menu.File_Open");

	file_options = new JMenuItem();
	file_options.addActionListener(Gatherer.g_man);
	file_options.setMnemonic(KeyEvent.VK_P);
	Dictionary.registerText(file_options, "Menu.File_Options");

	file_save = new JMenuItem();
	file_save.addActionListener(Gatherer.g_man);
	file_save.setEnabled(false);
	file_save.setMnemonic(KeyEvent.VK_S);
	Dictionary.registerText(file_save, "Menu.File_Save");

	// file_save_as = new JMenuItem();
	// file_save_as.addActionListener(Gatherer.g_man);
	// file_save_as.setEnabled(false);
	// file_save_as.setMnemonic(KeyEvent.VK_A);
	// Dictionary.registerText(file_save_as, "Menu.File_Save_As");

	// Layout (file menu)
	file.add(file_new);
	file.add(file_open);
	file.add(file_save);
	// file.add(file_save_as);
	file.add(file_close);
	file.add(new JSeparator());
	file.add(file_delete);
	file.add(file_export);
	file.add(new JSeparator());
	file.add(file_associations);
	file.add(file_options);
	file.add(new JSeparator());
	file.add(file_exit);

	// Edit menu
	edit = new JMenu();
	edit.setMnemonic(KeyEvent.VK_E);
	Dictionary.registerText(edit, "Menu.Edit");
		  
	edit_cut = new JMenuItem();
	edit_cut.addActionListener(Gatherer.g_man);
	edit_cut.setMnemonic(KeyEvent.VK_X);
	Dictionary.registerText(edit_cut, "Menu.Edit_Cut");

	edit_copy = new JMenuItem();
	edit_copy.addActionListener(Gatherer.g_man);
	edit_copy.setMnemonic(KeyEvent.VK_C);
	Dictionary.registerText(edit_copy, "Menu.Edit_Copy");

	edit_paste = new JMenuItem();
	edit_paste.addActionListener(Gatherer.g_man);
	edit_paste.setMnemonic(KeyEvent.VK_V);
	Dictionary.registerText(edit_paste, "Menu.Edit_Paste");

	// edit_undo = new JMenuItem();
	// edit_undo.setEnabled(false);
	// edit_undo.setMnemonic(KeyEvent.VK_U);
	// Dictionary.registerText(edit_undo, "Menu.Edit_Undo");

	// edit_redo = new JMenuItem();
	// edit_redo.setEnabled(false);
	// edit_redo.setMnemonic(KeyEvent.VK_D);
	// Dictionary.registerText(edit_redo, "Menu.Edit_Redo");

	// edit_search = new JMenuItem();
	// edit_search.addActionListener(Gatherer.g_man);
	// edit_search.setMnemonic(KeyEvent.VK_S);
	// Dictionary.registerText(edit_search, "Menu.Edit_Search");

	// edit_replace = new JMenuItem();
	// edit_replace.addActionListener(Gatherer.g_man);
	// edit_replace.setMnemonic(KeyEvent.VK_R);
	// Dictionary.registerText(edit_replace, "Menu.Edit_Replace");

	// Layout (edit menu)
	edit.add(edit_cut);
	edit.add(edit_copy);
	edit.add(edit_paste);
	// edit.add(new JSeparator());
	// edit.add(edit_undo);
	// edit.add(edit_redo);
	// edit.add(new JSeparator());
	// edit.add(edit_search);
	// edit.add(edit_replace);

	// Metadata menu
	metadata = new JMenu();
	metadata.setEnabled(false);
	metadata.setMnemonic(KeyEvent.VK_M);
	Dictionary.registerText(metadata, "Menu.Metadata");

	metadata_import = new JMenuItem();
	metadata_import.addActionListener(Gatherer.g_man);
	metadata_import.setMnemonic(KeyEvent.VK_I);
	Dictionary.registerText(metadata_import, "Menu.Metadata_Import");

	metadata_edit = new JMenuItem();
	metadata_edit.addActionListener(Gatherer.g_man);
	metadata_edit.setMnemonic(KeyEvent.VK_E);
	Dictionary.registerText(metadata_edit, "Menu.Metadata_Edit");

	metadata_export = new JMenuItem();
	metadata_export.addActionListener(Gatherer.g_man);
	metadata_export.setMnemonic(KeyEvent.VK_X);
	Dictionary.registerText(metadata_export, "Menu.Metadata_Export");

	metadata_view = new MagicMenuItem(Dictionary.get("Menu.Metadata_View") + " " + Dictionary.get("FileActions.No_Selection"), KeyEvent.VK_A);
	metadata_view.addActionListener(Gatherer.g_man);	

	// Layout (metadata sets menu)
	metadata.add(metadata_import);
	metadata.add(metadata_edit);
	metadata.add(metadata_export);
	metadata.add(metadata_view);

	// Tools menu
	// tools = new JMenu();
	// tools.setEnabled(false);
	// tools.setMnemonic(KeyEvent.VK_T);
	// Dictionary.registerText(tools, "Menu.Tools");

	// tools_log = new JMenuItem();
	// tools_log.addActionListener(Gatherer.g_man);
	// tools_log.setMnemonic(KeyEvent.VK_L);
	// Dictionary.registerText(tools_log, "Menu.Tools_Log");

	// tools_size = new JMenuItem();
	// tools_size.addActionListener(Gatherer.g_man);
	// tools_size.setMnemonic(KeyEvent.VK_C);
	// Dictionary.registerText(tools_size, "Menu.Tools_Calculate_Record_Size");

	// Layout (tools menu)
	// tools.add(tools_log);
	// tools.add(tools_size);

	// Help menu
	help = new JMenu();
	help.setIcon(Utility.HELP_ICON);
	Dictionary.setText(help, "Menu.Help");

	help_about = new JMenuItem();
	help_about.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_about, "Menu.Help_About");

	help_browse = new JMenuItem(Utility.BLANK_ICON);
	help_browse.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_browse, "GUI.Hunt");

	help_build = new JMenuItem(Utility.BLANK_ICON);
	help_build.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_build, "GUI.Create");

	help_collect = new JMenuItem(Utility.BLANK_ICON);
	help_collect.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_collect, "GUI.Gather");

	help_design = new JMenuItem(Utility.BLANK_ICON);
	help_design.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_design, "GUI.Design");

	help_export = new JMenuItem(Utility.BLANK_ICON);
	help_export.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_export, "GUI.Export");

	help_general = new JMenuItem();
	help_general.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_general, "Source.General");

	help_metaedit = new JMenuItem(Utility.BLANK_ICON);
	help_metaedit.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_metaedit, "GUI.Enrich");

	help_mirror = new JMenuItem(Utility.BLANK_ICON);
	help_mirror.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_mirror, "GUI.Mirror");

	help_preview = new JMenuItem(Utility.BLANK_ICON);
	help_preview.addActionListener(Gatherer.g_man);
	Dictionary.registerText(help_preview, "GUI.Preview");

	// Layout (help menu)
	help.add(help_general);
	help.add(new JSeparator());
	if (Gatherer.config.get("workflow.browse", true)) {
	    help.add(help_browse);
	}
	if (Gatherer.config.get("workflow.mirror", true)) {
	    help.add(help_mirror);
	}
	if (Gatherer.config.get("workflow.gather", true)) {
	    help.add(help_collect);
	}
	if (Gatherer.config.get("workflow.enrich", true)) {
	    help.add(help_metaedit);
	}
	if (Gatherer.config.get("workflow.design", true)) {
	    help.add(help_design);
	}
	if (Gatherer.config.get("workflow.export", true)) {
	    help.add(help_export);
	}
	if (Gatherer.config.get("workflow.create", true)) {
	    help.add(help_build);
	}
	if (Gatherer.config.get("workflow.preview", true)) {
	    help.add(help_preview);
	}
	help.add(new JSeparator());
	help.add(help_about);

	// Layout (menu bar)
	this.add(file);
	this.add(Box.createHorizontalStrut(15));
	this.add(edit);
	this.add(Box.createHorizontalStrut(15));
	this.add(metadata);
	// this.add(Box.createHorizontalStrut(15));
	// this.add(tools);
	this.add(Box.createHorizontalGlue());
	this.add(help);
    }

    public void collectionChanged(boolean ready) {
	file_close.setEnabled(ready);
	file_save.setEnabled(ready);
	// file_save_as.setEnabled(ready);
	metadata.setEnabled(ready);
	// tools.setEnabled(ready);
	if (ready) {
	    // Gatherer.c_man.undo.registerRedoSource(edit_redo);
	    // Gatherer.c_man.undo.registerUndoSource(edit_undo);
	}
    }

    /** Once a quit has been requested by the user, prevent any further menu selections. */
    public void exit() {
	file.setEnabled(false);
	edit.setEnabled(false);
	metadata.setEnabled(false);
	// tools.setEnabled(false);
	help.setEnabled(false);
    }

    public void setMetaAuditSuffix(String metaaudit_suffix) {
	///ystem.err.println("**** Set suffix: " + metaaudit_suffix);
	if(metaaudit_suffix == null) {
	    metadata_view.setText(Dictionary.get("Menu.Metadata_View") + " " + Dictionary.get("FileActions.No_Selection"));
	    metadata_view.setEnabled(false);
	}
	else {
	    ///ystem.err.println("Set metadata view suffix: " + metaaudit_suffix);
	    metadata_view.setText(Dictionary.get("Menu.Metadata_View") + " " + metaaudit_suffix);
	    metadata_view.setEnabled(true);
	}
    }

    /** Set the enabled state of one of the help menu items, based on its 'tabs' current state. Note that this method should only be called from the AWTEvent thread. */
    /* private void tabEnabled(int tab_index, boolean state) {
	JMenuItem selected = help.getItem(tab_index + 2); // Remember general and separator items
	selected.setEnabled(state);
	} */

    /** In order to provide context aware help advice we keep track of which
     *  tab the user has open, and then highlight that help menu item with
     *  separators.
     *  @param tab_index The index of the selected tab (0-7).
     */
    public void tabSelected(int tab_index) {
	JMenuItem selected;
	if(current_tab != -1) {
	    // Remove the image
	    selected = help.getItem(current_tab);
	    if(selected != null) {
		selected.setIcon(Utility.BLANK_ICON);
	    }
	}
	current_tab = tab_index + 2;
	selected = help.getItem(current_tab);
	if(selected != null) {
	    selected.setIcon(Utility.HELP_ICON);
	}
	selected = null;
    }


    public class MagicMenuItem
	extends JMenuItem {

	private boolean can_enable;
	private boolean should_enable;

	public MagicMenuItem(String title, int key_event) {
	    super(title, key_event);
	    super.setEnabled(false);
	    can_enable = false;
	    should_enable = false;
	}

	public void setCanEnable(boolean can_enable) {
	    this.can_enable = can_enable;
	    if (can_enable) {
		super.setEnabled(should_enable);
	    }
	    else {
		super.setEnabled(false);
	    }
	}

	public void setEnabled(boolean should_enable) {
	    this.should_enable = should_enable;
	    if (can_enable) {
		super.setEnabled(should_enable);
	    }
	}
    }
}
